@@ -1,8 +1,8 @@
 # pylint: disable=abstract-method
-""" # noqa: D205
-Representation of the POTCAR files
-----------------------------------
+"""
+Representation of the POTCAR files.
 
+-----------------------------------
 Attempt to create a convenient but licence-respecting storage system that also guarantees provenience.
 
 Consists of two classes, PotcarData and PotcarFileData. Between the two data node classes exists a
@@ -109,6 +109,7 @@ The mechanism for writing one or more PotcarData to file (from a calculation)::
             use Potcar.write_file
 
 """
+# pylint: disable=import-outside-toplevel
 from __future__ import print_function
 
 import re
@@ -119,7 +120,6 @@ import tempfile
 import shutil
 from contextlib import contextmanager
 from collections import namedtuple
-import six
 
 from py import path as py_path  # pylint: disable=no-name-in-module,no-member
 from pymatgen.io.vasp import PotcarSingle
@@ -269,13 +269,8 @@ class PotcarMetadataMixin(object):  # pylint: disable=useless-object-inheritance
         if not query_builder.count():
             raise NotExistent()
         results = [result[0] for result in query_builder.all()]
-        if six.PY3:
-            from functools import cmp_to_key
-            results.sort(key=cmp_to_key(by_older))
-            #results.sort(key=cmp_to_key(by_user))
-        else:
-            results.sort(by_older)
-            #results.sort(by_user)
+        from functools import cmp_to_key
+        results.sort(key=cmp_to_key(by_older))
         return results
 
     @classmethod
@@ -629,10 +624,13 @@ class PotcarData(Data, PotcarMetadataMixin, VersioningMixin):
         :param filter_symbols: list of strings with symbols to filter for.
         """
         group_query = QueryBuilder()
-        group_query.append(
-            Group, with_node='potcar_data', tag='potcar_data', filters={'type_string': {
-                '==': cls.potcar_family_type_string
-            }}, project='*')
+        group_query.append(Group,
+                           with_node='potcar_data',
+                           tag='potcar_data',
+                           filters={'type_string': {
+                               '==': cls.potcar_family_type_string
+                           }},
+                           project='*')
 
         groups = [group_list[0] for group_list in group_query.all()]
 
@@ -698,7 +696,7 @@ class PotcarData(Data, PotcarMetadataMixin, VersioningMixin):
             potcars_of_kind = [potcar[0] for potcar in query.all() if potcar[0].full_name == full_name]
             if not potcars_of_kind:
                 raise NotExistent('No POTCAR found for full name {} in family {}'.format(full_name, family_name))
-            elif len(potcars_of_kind) > 1:
+            if len(potcars_of_kind) > 1:
                 result_potcars[element] = cls.find(family=family_name, full_name=full_name)[0]
             else:
                 result_potcars[element] = potcars_of_kind[0]
@@ -773,9 +771,11 @@ class PotcarData(Data, PotcarMetadataMixin, VersioningMixin):
                 mapping=custom_mapping
             )
         """
-        # elements_to_name = {kind.symbol: kind.name for kind in structure.kinds}
         kind_names = structure.get_kind_names()
-        potcar_dict = {kind_name: potcar for kind_name, potcar in cls.get_potcars_dict(kind_names, family_name, mapping=mapping).items()}
+        potcar_dict = {kind_name: value
+                       for kind_name, value in cls.get_potcars_dict(kind_names, # pylint: disable=unnecessary-comprehension
+                                                                    family_name,
+                                                                    mapping=mapping).items()}  # yapf: disable
         return potcar_dict
 
     @classmethod
@@ -944,11 +944,6 @@ class PotcarData(Data, PotcarMetadataMixin, VersioningMixin):
         if not query.count():
             raise NotExistent()
         results = [result[0] for result in query.all()]
-        if six.PY3:
-            from functools import cmp_to_key
-            results.sort(key=cmp_to_key(by_older))
-            #results.sort(key=cmp_to_key(by_user))
-        else:
-            results.sort(by_older)
-            #results.sort(by_user)
+        from functools import cmp_to_key
+        results.sort(key=cmp_to_key(by_older))
         return results
