@@ -1,181 +1,153 @@
-""" # noqa: D205
-Relax workchain
----------------
-Structure relaxation workchain.
+"""
+Relax workchain.
+
+----------------
+Structure relaxation workchain, which performs the regular duties of relaxing a structure.
+It has been designed such that calling workchains should try to use human readable
+parameters instead of the code dependent variables.
 """
 # pylint: disable=attribute-defined-outside-init
-import enum
+import numpy as np
+
 from aiida.common.extendeddicts import AttributeDict
 from aiida.engine import WorkChain, append_, while_, if_
 from aiida.plugins import WorkflowFactory
 
 from aiida_vasp.utils.aiida_utils import get_data_class, get_data_node
 from aiida_vasp.utils.workchains import compare_structures, prepare_process_inputs, compose_exit_code
+from aiida_vasp.utils.parameters import inherit_and_merge_parameters
 
 
 class RelaxWorkChain(WorkChain):
     """Structure relaxation workchain."""
-
     _verbose = False
     _next_workchain_string = 'vasp.verify'
     _next_workchain = WorkflowFactory(_next_workchain_string)
 
-    class AlgoEnum(enum.IntEnum):
-        """Encode values for algorithm descriptively in enum."""
-        NO_UPDATE = -1
-        IONIC_RELAXATION_RMM_DIIS = 1
-        IONIC_RELAXATION_CG = 2
-
-    class ModeEnum(enum.IntEnum):
-        """Encode values for mode of relaxation descriptively in enum."""
-        POS_ONLY = 1
-        POS_SHAPE_VOL = 3
-        POS_SHAPE = 4
-        SHAPE_ONLY = 5
-
-        @classmethod
-        def get_from_dof(cls, **kwargs):
-            """Get the correct mode of relaxation for the given degrees of freedom."""
-            dof = tuple(kwargs[i] for i in ['positions', 'shape', 'volume'])
-            value_from_dof = {
-                (True, False, False): cls.POS_ONLY,
-                (True, True, True): cls.POS_SHAPE_VOL,
-                (True, True, False): cls.POS_SHAPE,
-                (False, True, False): cls.SHAPE_ONLY
-            }
-            return value_from_dof[dof]
-
     @classmethod
     def define(cls, spec):
         super(RelaxWorkChain, cls).define(spec)
-        spec.expose_inputs(cls._next_workchain, exclude=('parameters', 'structure', 'relax_parameters', 'settings'))
+        spec.expose_inputs(cls._next_workchain, exclude=('parameters', 'structure', 'settings'))
         spec.input('structure', valid_type=(get_data_class('structure'), get_data_class('cif')))
-        spec.input('parameters', valid_type=get_data_class('dict'), required=False)
-        spec.input('relax_parameters', valid_type=get_data_class('dict'), required=False)
+        spec.input('parameters', valid_type=get_data_class('dict'))
         spec.input('settings', valid_type=get_data_class('dict'), required=False)
-        spec.input(
-            'relax',
-            valid_type=get_data_class('bool'),
-            required=False,
-            default=get_data_node('bool', False),
-            help="""
+        spec.input('relax.perform',
+                   valid_type=get_data_class('bool'),
+                   required=False,
+                   default=lambda: get_data_node('bool', False),
+                   help="""
             If True, perform relaxation.
             """)
-        spec.input(
-            'energy_cutoff',
-            valid_type=get_data_class('float'),
-            required=False,
-            help="""
+        spec.input('relax.algo',
+                   valid_type=get_data_class('str'),
+                   default=lambda: get_data_node('str', 'cg'),
+                   help="""
+            The algorithm to use during relaxation.
+            """)
+        spec.input('relax.energy_cutoff',
+                   valid_type=get_data_class('float'),
+                   required=False,
+                   help="""
             The cutoff that determines when the relaxation procedure is stopped. In this
             case it stops when the total energy between two ionic steps is less than the
             supplied value.
             """)
-        spec.input(
-            'force_cutoff',
-            valid_type=get_data_class('float'),
-            required=False,
-            help="""
+        spec.input('relax.force_cutoff',
+                   valid_type=get_data_class('float'),
+                   required=False,
+                   help="""
             The cutoff that determines when the relaxation procedure is stopped. In this
             case it stops when all forces are smaller than than the
             supplied value.
             """)
-        spec.input(
-            'steps',
-            valid_type=get_data_class('int'),
-            required=False,
-            default=get_data_node('int', 60),
-            help="""
+        spec.input('relax.steps',
+                   valid_type=get_data_class('int'),
+                   required=False,
+                   default=lambda: get_data_node('int', 60),
+                   help="""
                    The number of relaxation steps to perform (updates to the atomic positions,
             unit cell size or shape).
                    """)
-        spec.input(
-            'positions',
-            valid_type=get_data_class('bool'),
-            required=False,
-            default=get_data_node('bool', True),
-            help="""
+        spec.input('relax.positions',
+                   valid_type=get_data_class('bool'),
+                   required=False,
+                   default=lambda: get_data_node('bool', True),
+                   help="""
                    If True, perform relaxation of the atomic positions.
                    """)
-        spec.input(
-            'shape',
-            valid_type=get_data_class('bool'),
-            required=False,
-            default=get_data_node('bool', False),
-            help="""
+        spec.input('relax.shape',
+                   valid_type=get_data_class('bool'),
+                   required=False,
+                   default=lambda: get_data_node('bool', False),
+                   help="""
                    If True, perform relaxation of the unit cell shape.
                    """)
-        spec.input(
-            'volume',
-            valid_type=get_data_class('bool'),
-            required=False,
-            default=get_data_node('bool', False),
-            help="""
+        spec.input('relax.volume',
+                   valid_type=get_data_class('bool'),
+                   required=False,
+                   default=lambda: get_data_node('bool', False),
+                   help="""
                    If True, perform relaxation of the unit cell volume..
                    """)
-        spec.input(
-            'convergence_on',
-            valid_type=get_data_class('bool'),
-            required=False,
-            default=get_data_node('bool', False),
-            help="""
+        spec.input('relax.convergence_on',
+                   valid_type=get_data_class('bool'),
+                   required=False,
+                   default=lambda: get_data_node('bool', False),
+                   help="""
                    If True, test convergence based on selected criterias set.
                    """)
-        spec.input(
-            'convergence_absolute',
-            valid_type=get_data_class('bool'),
-            required=False,
-            default=get_data_node('bool', False),
-            help="""
+        spec.input('relax.convergence_absolute',
+                   valid_type=get_data_class('bool'),
+                   required=False,
+                   default=lambda: get_data_node('bool', False),
+                   help="""
                    If True, test convergence based on absolute differences.
                    """)
-        spec.input(
-            'convergence_max_iterations',
-            valid_type=get_data_class('int'),
-            required=False,
-            default=get_data_node('int', 5),
-            help="""
+        spec.input('relax.convergence_max_iterations',
+                   valid_type=get_data_class('int'),
+                   required=False,
+                   default=lambda: get_data_node('int', 5),
+                   help="""
                    The number of iterations to perform if the convergence criteria is not met.
                    """)
-        spec.input(
-            'convergence_volume',
-            valid_type=get_data_class('float'),
-            required=False,
-            default=get_data_node('float', 0.01),
-            help="""
+        spec.input('relax.convergence_volume',
+                   valid_type=get_data_class('float'),
+                   required=False,
+                   default=lambda: get_data_node('float', 0.01),
+                   help="""
                    The cutoff value for the convergence check on volume. If ``convergence_absolute``
                    is True in AA, otherwise in relative.
                    """)
-        spec.input(
-            'convergence_positions',
-            valid_type=get_data_class('float'),
-            required=False,
-            default=get_data_node('float', 0.01),
-            help="""
+        spec.input('relax.convergence_positions',
+                   valid_type=get_data_class('float'),
+                   required=False,
+                   default=lambda: get_data_node('float', 0.01),
+                   help="""
                    The cutoff value for the convergence check on positions. If ``convergence_absolute``
                    is True in AA, otherwise in relative difference.
                    """)
-        spec.input(
-            'convergence_shape_lengths',
-            valid_type=get_data_class('float'),
-            required=False,
-            default=get_data_node('float', 0.1),
-            help="""
+        spec.input('relax.convergence_shape_lengths',
+                   valid_type=get_data_class('float'),
+                   required=False,
+                   default=lambda: get_data_node('float', 0.1),
+                   help="""
                    The cutoff value for the convergence check on the lengths of the unit cell
                    vecotrs. If ``convergence_absolute``
                    is True in AA, otherwise in relative difference.
                    """)
-        spec.input(
-            'convergence_shape_angles',
-            valid_type=get_data_class('float'),
-            required=False,
-            default=get_data_node('float', 0.1),
-            help="""
+        spec.input('relax.convergence_shape_angles',
+                   valid_type=get_data_class('float'),
+                   required=False,
+                   default=lambda: get_data_node('float', 0.1),
+                   help="""
                    The cutoff value for the convergence check on the angles of the unit cell.
                    If ``convergence_absolute`` is True in degrees, otherwise in relative difference.
                    """)
         spec.exit_code(0, 'NO_ERROR', message='the sun is shining')
-        spec.exit_code(
-            300, 'ERROR_MISSING_REQUIRED_OUTPUT', message='the called workchain does not contain the necessary relaxed output structure')
+        spec.exit_code(300,
+                       'ERROR_MISSING_REQUIRED_OUTPUT',
+                       message='the called workchain does not contain the necessary relaxed output structure')
+        spec.exit_code(420, 'ERROR_NO_CALLED_WORKCHAIN', message='no called workchain detected')
         spec.exit_code(500, 'ERROR_UNKNOWN', message='unknown error detected in the relax workchain')
         spec.exit_code(502, 'ERROR_OVERRIDE_PARAMETERS', message='there was an error overriding the parameters')
         spec.outline(
@@ -198,68 +170,7 @@ class RelaxWorkChain(WorkChain):
         )  # yapf: disable
 
         spec.expose_outputs(cls._next_workchain)
-        spec.output('structure_relaxed', valid_type=get_data_class('structure'), required=False)
-
-    def _set_ibrion(self, parameters):
-        """Set the algorithm to use for relaxation."""
-        if self.inputs.positions.value or self.inputs.shape.value or self.inputs.volume.value:
-            parameters.ibrion = self.AlgoEnum.IONIC_RELAXATION_CG
-        else:
-            parameters.ibrion = self.AlgoEnum.NO_UPDATE
-
-    def _set_ediffg(self, parameters):
-        """Set the cutoff to use for relaxation."""
-        energy_cutoff = False
-        try:
-            parameters.ediffg = self.inputs.energy_cutoff.value
-            energy_cutoff = True
-        except AttributeError:
-            pass
-        try:
-            parameters.ediffg = -abs(self.inputs.force_cutoff.value)
-            if energy_cutoff:
-                self.report('User supplied both a force and an energy cutoff for the relaxation. Utilizing the force cutoff.')
-        except AttributeError:
-            pass
-
-    def _set_nsw(self, parameters):
-        """Set the number of ionic steps to perform."""
-        parameters.nsw = self.inputs.steps.value
-
-    def _set_isif(self, parameters):
-        """Set relaxation mode according to the chosen degrees of freedom."""
-        parameters.isif = self.ModeEnum.get_from_dof(
-            positions=self.inputs.positions.value, shape=self.inputs.shape.value, volume=self.inputs.volume.value)
-
-    def _add_overrides(self, parameters):
-        """Add parameters tag overrides, except the ones controlled by other inputs (for provenance)."""
-        overrides = AttributeDict({k.lower(): v for k, v in self.inputs.relax_parameters.get_dict().items()})
-        parameters.update(overrides)
-
-    def _init_parameters(self):
-        """Set parameters parameters based on other inputs."""
-        parameters = AttributeDict()
-        try:
-            input_parameters = self.inputs.parameters.get_dict()
-            check_parameters_relax_entries(input_parameters)
-            parameters.update(input_parameters)
-        except AttributeError:
-            pass
-        if self.perform_relaxation():
-            if 'relax_parameters' in self.inputs:
-                # Add override parameters (user force what to use)
-                try:
-                    self._add_overrides(parameters)
-                except ValueError as err:
-                    return compose_exit_code(self.exit_code.ERROR_OVERRIDE_PARAMETERS, str(err))
-            else:
-                # Add plugin controlled flags
-                self._set_ibrion(parameters)
-                self._set_isif(parameters)
-                self._set_nsw(parameters)
-                self._set_ediffg(parameters)
-
-        return parameters
+        spec.output('relax.structure', valid_type=get_data_class('structure'), required=False)
 
     def initialize(self):
         """Initialize."""
@@ -272,10 +183,20 @@ class RelaxWorkChain(WorkChain):
         """Store exposed inputs in the context."""
         self.ctx.exit_code = self.exit_codes.ERROR_UNKNOWN  # pylint: disable=no-member
         self.ctx.is_converged = False
+        self.ctx.relax = False
         self.ctx.iteration = 0
         self.ctx.workchains = []
         self.ctx.inputs = AttributeDict()
 
+    def _init_inputs(self):
+        """Initialize the inputs."""
+        self.ctx.inputs.parameters = self._init_parameters()
+        try:
+            self._verbose = self.inputs.verbose.value
+            self.ctx.inputs.verbose = self.inputs.verbose
+        except AttributeError:
+            pass
+
     def _init_structure(self):
         """Initialize the structure."""
         self.ctx.current_structure = self.inputs.structure
@@ -296,34 +217,46 @@ class RelaxWorkChain(WorkChain):
                 settings.parser_settings = dict_entry
         self.ctx.inputs.settings = settings
 
-    def _init_inputs(self):
-        """Initialize the inputs."""
-        self.ctx.inputs.parameters = self._init_parameters()
-        try:
-            self._verbose = self.inputs.verbose.value
-        except AttributeError:
-            pass
+    def _init_parameters(self):
+        """Collect input to the workchain in the relax namespace and put that into the parameters."""
+
+        # At some point we will replace this with possibly input checking using the PortNamespace on
+        # a dict parameter type. As such we remove the workchain input parameters as node entities. Much of
+        # the following is just a workaround until that is in place in AiiDA core.
+        parameters = inherit_and_merge_parameters(self.inputs)
+
+        # Need to save the parameter that controls if relaxation should be performed
+        self.ctx.relax = parameters.relax.perform
+        if not parameters.relax.perform:
+            # Make sure we do not expose the relax namespace in the input parameters (
+            # basically setting no code tags related to relaxation unless user overrides)
+            del parameters.relax
+
+        return parameters
 
     def _set_default_relax_settings(self):
         """Set default settings."""
 
     def run_next_workchains(self):
-        within_max_iterations = bool(self.ctx.iteration < self.inputs.convergence_max_iterations)
+        within_max_iterations = bool(self.ctx.iteration < self.inputs.relax.convergence_max_iterations)
         return bool(within_max_iterations and not self.ctx.is_converged)
 
     def init_relaxed(self):
         """Initialize a calculation based on a relaxed or assumed relaxed structure."""
         if not self.perform_relaxation():
             if self._verbose:
-                self.report('skipping structure relaxation and forwarding input/output to the next workchain.')
+                self.report('skipping structure relaxation and forwarding input to the next workchain.')
         else:
             # For the final static run we do not need to parse the output structure, which
             # is at this point enabled.
-            settings = AttributeDict(self.ctx.inputs.settings.get_dict())
-            settings.parser_settings['add_structure'] = False
-            self.ctx.inputs.settings = settings
+            self.ctx.inputs.settings.parser_settings.add_structure = False
             try:
-                self.ctx.inputs.parameters = self.inputs.parameters
+                self.ctx.inputs.settings.parser_settings.add_structure = False
+            except AttributeError:
+                pass
+            # Remove relaxation parameters if they exist
+            try:
+                del self.ctx.inputs.parameters.relax
             except AttributeError:
                 pass
             if self._verbose:
@@ -346,12 +279,12 @@ class RelaxWorkChain(WorkChain):
         # Add exposed inputs
         self.ctx.inputs.update(self.exposed_inputs(self._next_workchain))
 
-        # Make sure we do not have any floating dict (convert to Dict)
-        self.ctx.inputs = prepare_process_inputs(self.ctx.inputs)
+        # Make sure we do not have any floating dict (convert to Dict etc.)
+        self.ctx.inputs_ready = prepare_process_inputs(self.ctx.inputs, namespaces=['verify'])
 
     def run_next_workchain(self):
         """Run the next workchain."""
-        inputs = self.ctx.inputs
+        inputs = self.ctx.inputs_ready
         running = self.submit(self._next_workchain, **inputs)
 
         if not self.ctx.is_converged and self.perform_relaxation():
@@ -402,16 +335,16 @@ class RelaxWorkChain(WorkChain):
         self.ctx.current_structure = workchain.outputs.structure
 
         converged = True
-        if self.inputs.convergence_on.value:
+        if self.ctx.inputs.parameters.relax.convergence_on:
             if self._verbose:
                 self.report('Checking the convergence of the relaxation.')
             comparison = compare_structures(self.ctx.previous_structure, self.ctx.current_structure)
-            delta = comparison.absolute if self.inputs.convergence_absolute.value else comparison.relative
-            if self.inputs.positions.value:
+            delta = comparison.absolute if self.ctx.inputs.parameters.relax.convergence_absolute else comparison.relative
+            if self.ctx.inputs.parameters.relax.positions:
                 converged &= self.check_positions_convergence(delta)
-            if self.inputs.volume.value:
+            if self.ctx.inputs.parameters.relax.volume:
                 converged &= self.check_volume_convergence(delta)
-            if self.inputs.shape.value:
+            if self.ctx.inputs.parameters.relax.shape:
                 converged &= self.check_shape_convergence(delta)
 
         if not converged:
@@ -427,29 +360,44 @@ class RelaxWorkChain(WorkChain):
 
     def check_shape_convergence(self, delta):
         """Check the difference in cell shape before / after the last iteratio against a tolerance."""
-        lengths_converged = bool(delta.cell_lengths.max() <= self.inputs.convergence_shape_lengths.value)
+        lengths_converged = bool(delta.cell_lengths.max() <= self.ctx.inputs.parameters.relax.convergence_shape_lengths)
         if not lengths_converged:
-            self.report('cell lengths changed by max {}, tolerance is {}'.format(delta.cell_lengths.max(),
-                                                                                 self.inputs.convergence_shape_lengths.value))
+            self.report('cell lengths changed by max {}, tolerance is {}'.format(
+                delta.cell_lengths.max(), self.ctx.inputs.parameters.relax.convergence_shape_lengths))
 
-        angles_converged = bool(delta.cell_angles.max() <= self.inputs.convergence_shape_angles.value)
+        angles_converged = bool(delta.cell_angles.max() <= self.ctx.inputs.parameters.relax.convergence_shape_angles)
         if not angles_converged:
             self.report('cell angles changed by max {}, tolerance is {}'.format(delta.cell_angles.max(),
-                                                                                self.inputs.convergence_shape_angles.value))
+                                                                                self.ctx.inputs.parameters.relax.convergence_shape_angles))
 
         return bool(lengths_converged and angles_converged)
 
     def check_volume_convergence(self, delta):
-        volume_converged = bool(delta.volume <= self.inputs.convergence_volume.value)
+        """Check the convergence of the volume, given a cutoff."""
+        volume_converged = bool(delta.volume <= self.ctx.inputs.parameters.relax.convergence_volume)
         if not volume_converged:
-            self.report('cell volume changed by {}, tolerance is {}'.format(delta.volume, self.inputs.convergence_volume.value))
+            self.report('cell volume changed by {}, tolerance is {}'.format(delta.volume,
+                                                                            self.ctx.inputs.parameters.relax.convergence_volume))
         return volume_converged
 
     def check_positions_convergence(self, delta):
-        positions_converged = bool(delta.pos_lengths.max() <= self.inputs.convergence_positions.value)
+        """Check the convergence of the atomic positions, given a cutoff."""
+        try:
+            positions_converged = bool(np.nanmax(delta.pos_lengths) <= self.ctx.inputs.parameters.relax.convergence_positions)
+        except RuntimeWarning:
+            # Here we encountered the case of having one atom centered at the origin, so
+            # we do not know if it is converged, so settings it to False
+            self.report('there is NaN entries in the relative comparison for '
+                        'the positions during relaxation, assuming position is not converged')
+            positions_converged = False
+
         if not positions_converged:
-            self.report('max site position change is {}, tolerance is {}'.format(delta.pos_lengths.max(),
-                                                                                 self.inputs.convergence_positions.value))
+            try:
+                self.report('max site position change is {}, tolerance is {}'.format(np.nanmax(
+                    delta.pos_lengths), self.ctx.inputs.parameters.relax.convergence_positions))
+            except RuntimeWarning:
+                pass
+
         return positions_converged
 
     def store_relaxed(self):
@@ -459,8 +407,8 @@ class RelaxWorkChain(WorkChain):
         relaxed_structure = workchain.outputs.structure
         if self._verbose:
             self.report("attaching the node {}<{}> as '{}'".format(relaxed_structure.__class__.__name__, relaxed_structure.pk,
-                                                                   'structure_relaxed'))
-        self.out('structure_relaxed', relaxed_structure)
+                                                                   'relax.structure'))
+        self.out('relax.structure', relaxed_structure)
 
     def results(self):
         """Attach the remaining output results."""
@@ -473,16 +421,4 @@ class RelaxWorkChain(WorkChain):
 
     def perform_relaxation(self):
         """Check if a relaxation is to be performed."""
-        return self.inputs.relax.value
-
-
-def check_parameters_relax_entries(parameters):
-    """Check that some relaxation flags are not present in the parameters (no override is allowed)."""
-
-    overrides = AttributeDict({k.lower(): v for k, v in parameters.items()})
-    if 'ibrion' in overrides:
-        raise ValueError('overriding IBRION not allowed, use inputs to control')
-    if 'isif' in overrides:
-        raise ValueError('overriding ISIF not allowed, use inputs to control')
-    if 'nsw' in overrides:
-        raise ValueError('overriding NSW not allowed, use inputs.steps to control')
+        return self.ctx.relax
